from typing import Annotated, Optional

from asyncreq import request
from fastapi import APIRouter, Header, HTTPException

from src.api.config import API_FLIGHTS
from src.api.routes.auth import status as checkAuth
from src.api.schemas.flight import Flight, FlightCreate, FlightUpdate

router = APIRouter()


@router.get("/{id}", response_model=Flight)
async def get_flight_by_id(id: int):
    (response, status, _) = await request(f"{API_FLIGHTS}/{id}", "GET")
    if status < 200 or status > 204:
        raise HTTPException(status_code=status, detail=response)
    return response


@router.post("", response_model=Flight)
async def create_flight(
    flight: FlightCreate, authorization: Annotated[str | None, Header()] = None
):
    auth = await checkAuth(authorization)
    flight_data = flight.model_dump()
    flight_data["user_id"] = auth["id"]
    (response, status, _) = await request(f"{API_FLIGHTS}", "POST", json=flight_data)
    if status < 200 or status > 204:
        raise HTTPException(status_code=status, detail=response)
    return response


@router.patch("/{id}", response_model=Flight)
async def update_flight(
    id: int,
    flight_update: FlightUpdate,
    authorization: Annotated[str | None, Header()] = None,
):
    auth = await checkAuth(authorization)
    update = flight_update.model_dump()
    update["user_id"] = auth["id"]
    (response, status, _) = await request(f"{API_FLIGHTS}/{id}", "PATCH", json=update)
    if status < 200 or status > 204:
        raise HTTPException(status_code=status, detail=response)
    return response


@router.get("", response_model=list[Flight])
async def get_flights(
    origin: Optional[str] = None,
    destination: Optional[str] = None,
    lastUpdated: Optional[str] = None,
    future: Optional[str] = None,
):
    query = {}
    if origin:
        query["origin"] = origin
    if destination:
        query["destination"] = destination
    if lastUpdated:
        query["lastUpdated"] = lastUpdated
    if future:
        query["future"] = future
    (response, status, _) = await request(f"{API_FLIGHTS}", "GET", query=query)
    if status < 200 or status > 204:
        raise HTTPException(status_code=status, detail=response)
    return response
